==> snowlog-1.0.3.tar.gz: snowlog-1.0.3/src/snowtk.c <==
/* Snowlog - An Apache logfile browser/analyzer.
 * 
 * Copyright 2004 Oliver Feiler <kiza@kcore.de>
 * http://snowlog.kcore.de/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as   
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */
   
#include <ncurses.h>

#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "snowtk.h"

struct snow_stringhash_item {
    struct snow_stringhash_item *next;
    char *string;
};

struct snow_stringhash {
    struct snow_stringhash_item **table;
    unsigned table_size;
};

extern char *browser;

/* Init the ncurses library. */
void InitCurses (void) {
	initscr();
	
	keypad (stdscr, TRUE);	/* Activate keypad so we can read function keys with getch. */
	
	cbreak();				/* No buffering. */
	noecho();				/* Do not echo typed chars onto the screen. */
	clear();
	curs_set(0);			/* Hide cursor. */
	refresh();
}

void ShutdownCurses (void) {
	clear();
	refresh();
	endwin();
}

/* Print text in statusbar. */
void UIStatus (const char * text, int delay, int warning, int ypos) {
	int i;
	
	attron (WA_REVERSE);
	if (warning)
		attron (COLOR_PAIR(10));
	
	/* Clear statusbar. */
	for (i = 0; i < COLS; i++) {
		mvaddch (ypos, i, ' ');
	}
	
	mvaddnstr (ypos, 1, text, COLS-2);
	
	/* attroff is called here. If the calling function had it switched on,
	   switch it on again there! */
	if (warning)
		attroff (COLOR_PAIR(10));
	attroff (WA_REVERSE);
		
	refresh();
	
	if (delay)
		sleep (delay);
}

/* Draw a box with WA_REVERSE at coordinates x1y1/x2y2 */
void UISupportDrawBox (int x1, int y1, int x2, int y2) {
	int i, j;
	
	attron (WA_REVERSE);
	for (i = y1; i <= y2; i++) {
		for (j = x1; j <= x2; j++) {
				mvaddch (i, j, ' ');
		}	
	}
	attroff (WA_REVERSE);
}

/* Draw main program header. */
void UISupportDrawHeader (const char * headerstring) {
	UISupportDrawBox(0, 0, COLS, 0);
	attron (WA_REVERSE);

	mvprintw (0, 1, "* Snowlog %s", VERSION);
	if (headerstring != NULL) {
		if (strlen(headerstring) > COLS-18)
			mvaddnstr (0, 19, headerstring, COLS-20);
		else
			mvaddstr (0, COLS-strlen(headerstring)-1, headerstring);
	}
	attroff (WA_REVERSE);
}

void SnowClearScreen (int * forceredraw) {
	if (*forceredraw) {
		clear();
		*forceredraw = 0;
	} else {
		move(0,0);
		clrtobot();
	}
}

void SnowClearLine (int y) {
	move (y, 0);
	clrtoeol();
}

void SnowStartColors (void) {
	start_color();

	/* The following call will automagically implement -1 as the terminal's
	default color for fg/bg in init_pair. */
	use_default_colors();

	/* Internally used color pairs.
	Use with WA_BOLD to get bright color (orange->yellow, gray->white, etc). */
	init_pair (SNOW_COLOR_RED, 1, -1);		/* red */
	init_pair (SNOW_COLOR_GREEN, 2, -1);	/* green */
	init_pair (SNOW_COLOR_ORANGE, 3, -1);	/* orange */
	init_pair (SNOW_COLOR_BLUE, 4, -1);		/* blue */
	init_pair (SNOW_COLOR_MAGENTA, 5, -1);	/* magenta */
	init_pair (SNOW_COLOR_CYAN, 6, -1);		/* cyan */
	init_pair (SNOW_COLOR_GREY, 7, -1);		/* gray */

	/* Default terminal color color pair */
	init_pair (63, -1, -1);
}

/* Take a URL and execute in default browser.
   Apply all security restrictions before running system()! */
void UISupportURLJump (const char * url) {
	int len;
	char *tmp;
	char *escapetext = NULL;
	char *syscall = NULL;
	
	/* Should not happen. Nope, really should not happen. */
	if ((url == NULL) || (strcmp(url, "-") == 0))
		return;
	
	/* Complain loudly if browser string contains a single quote. */
	if (strstr(browser, "'") != NULL)
		UIStatus ("Unsafe browser string (contains quotes)! See snownews.kcore.de/faq#toc2.4", 5, 1, LINES-1);
	
	/* Discard url if it contains a single quote! */
	if ((strstr(url, "'")) == NULL) {
		len = strlen(url) + 3;
		escapetext = malloc (len);
		snprintf (escapetext, len, "\'%s\'", url);
		len = strlen(browser) + len + 1;
		syscall = malloc (len);
		snprintf (syscall, len, browser, escapetext);
		free (escapetext);
	} else {
		UIStatus ("Invalid URL passed. Possible shell exploit attempted!", 3, 1, LINES-1);
		free (syscall);
		return;
	}
	
	len = strlen(syscall) + strlen("Executing %s") + 1;
	tmp = malloc (len);
	snprintf (tmp, len, "Executing %s", syscall);
	UIStatus (tmp, 0, 0, LINES-1);
	free (tmp);
	
	len = len + 13;
	tmp = malloc (len);
	snprintf (tmp, len, "%s 2>/dev/null", syscall);

	/* Switch on the cursor. */
	curs_set(1);

	system(tmp);

	InitCurses();

	/* Hide cursor again. */
	curs_set(0);

	free (tmp);
	free (syscall);
}

void Help (void) {
	int uiinput;
	int centerx, centery;
	int offset = 22;
	int offsetstr = 0;
	
	centerx = COLS / 2;
	centery = LINES / 2;
	
	UISupportDrawBox ((COLS/2)-24, (LINES/2)-6, (COLS/2)+24, (LINES/2)+5);
	
	attron(WA_REVERSE);
	mvaddstr (centery-5, COLS/2-strlen("Keyboard commands")/2, "Keyboard commands");
	mvprintw (centery-3, centerx-offset, "%s:", "tab");
	mvprintw (centery-2, centerx-offset, "%s:", "n, p, up, down");
	mvprintw (centery-1, centerx-offset, "%s:", "N, P, <-, ->");
	mvprintw (centery, centerx-offset, "%s:", "space, b, pdown, pup");
	mvprintw (centery+1, centerx-offset, "%s:", "<, >, home, end");
	mvprintw (centery+2, centerx-offset, "%c:", 'o');
	mvprintw (centery+3, centerx-offset, "%c:", 'O');
	mvprintw (centery+4, centerx-offset, "%c:", 'f');
	
	mvaddstr (centery-3, centerx-offsetstr, "Search");
	mvaddstr (centery-2, centerx-offsetstr, "Next, previous item");
	mvaddstr (centery-1, centerx-offsetstr, "Next, previous host");
	mvaddstr (centery, centerx-offsetstr, "Next, previous page");
	mvaddstr (centery+1, centerx-offsetstr, "First, last item");
	mvaddstr (centery+2, centerx-offsetstr, "Open referer in browser");
	mvaddstr (centery+3, centerx-offsetstr, "Open current resource");
	mvaddstr (centery+4, centerx-offsetstr, "Filters");
	attroff(WA_REVERSE);

	UIStatus ("Press the any(tm) key to exit help screen.", 0, 0, LINES-1);
	uiinput = getch();
	
	/* Return input back into input queue so it gets automatically
	   executed. */
	if ((uiinput != '\n') && (uiinput != 'h') && (uiinput != 'q'))
		ungetch(uiinput);
}

static inline unsigned snow_stringhash__hashvalue(char const * const string)
{
    unsigned len = strlen(string);
    unsigned i, h = 0;
    for (i=0; i<len; i++) {
        h = 31*h + string[i];
    }
    return h;
} 

struct snow_stringhash *snow_stringhash_init(unsigned const table_size)
{
    struct snow_stringhash *newhash = malloc(sizeof(struct snow_stringhash));
    newhash->table_size = table_size;
    newhash->table = malloc(sizeof(struct snow_stringhash_item *)*table_size);
    memset(newhash->table, 0, sizeof(struct snow_stringhash_item *)*table_size);
    return newhash;
}

void snow_stringhash_free(struct snow_stringhash * const hash)
{
    struct snow_stringhash_item **table_ptr = hash->table;
    struct snow_stringhash_item *cur_item, *next_item;
    unsigned i;

    /* Iterate through the whole table and free all stored items */
    for (i = 0, table_ptr = hash->table;
         i < hash->table_size; 
         i++, table_ptr++) 
    {
        for (cur_item = *table_ptr; cur_item != NULL; cur_item = next_item) {
            next_item = cur_item->next;
            free(cur_item->string);
            free(cur_item);
        }
    }
    
    free(hash->table);
    free(hash);
}

char const * snow_stringhash_unique(struct snow_stringhash * const hash,
                                    char const * const string)
{
    struct snow_stringhash_item **table_ptr;
    struct snow_stringhash_item *cur_item, *new_item;
    
    table_ptr = hash->table + (snow_stringhash__hashvalue(string) % hash->table_size);
    
    // Check if the string is already in the hash
    for (cur_item=*table_ptr; cur_item != NULL; cur_item = cur_item->next) {
        if (strcmp(cur_item->string, string) == 0)
            return cur_item->string;
    }
    
    // String is not in the hash, add it
    new_item = malloc(sizeof(struct snow_stringhash_item));
    new_item->next = *table_ptr;
    new_item->string = strdup(string);
    *table_ptr = new_item;
    
    return new_item->string;
}

void snow_stringhash_dump(struct snow_stringhash * const hash)
{
    struct snow_stringhash_item **table_ptr = hash->table;
    struct snow_stringhash_item *cur_item;
    unsigned *bucket_sizes;
    unsigned cur_bucket;
    unsigned item_count = 0;
    
    bucket_sizes = malloc(sizeof(unsigned) * hash->table_size);
    memset(bucket_sizes, 0, sizeof(unsigned) * hash->table_size);
    
    /* Iterate through the whole table and collect the bucket sizes */
    for (cur_bucket = 0, table_ptr = hash->table;
         cur_bucket < hash->table_size; 
         cur_bucket++, table_ptr++) 
    {
        for (cur_item = *table_ptr; cur_item != NULL; cur_item = cur_item->next) {
            bucket_sizes[cur_bucket]++;
            item_count++;
        }
        fprintf(stderr, "Size of bucket %4d = %d: ", cur_bucket, bucket_sizes[cur_bucket]);
        for (cur_item = *table_ptr; cur_item != NULL; cur_item = cur_item->next) {
        	fprintf(stderr, "%s, ", cur_item->string);
        }
        fprintf(stderr, "\n");
    }
    
    free(bucket_sizes);
}

==> snowlog-1.1.1.tar.gz: snowlog-1.1.1/src/snowtk.c <==
/* Snowlog - An Apache logfile browser/analyzer.
 * 
 * Copyright 2004 Oliver Feiler <kiza@kcore.de>
 * http://snowlog.kcore.de/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as   
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */
   
#include <ncurses.h>

#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "snowtk.h"

struct snow_stringhash_item {
    struct snow_stringhash_item *next;
    char *string;
};

struct snow_stringhash {
    struct snow_stringhash_item **table;
    unsigned table_size;
};

extern char *browser;

/* Init the ncurses library. */
void InitCurses (void) {
	initscr();
	
	keypad (stdscr, TRUE);	/* Activate keypad so we can read function keys with getch. */
	
	cbreak();				/* No buffering. */
	noecho();				/* Do not echo typed chars onto the screen. */
	clear();
	curs_set(0);			/* Hide cursor. */
	refresh();
}

void ShutdownCurses (void) {
	clear();
	refresh();
	endwin();
}

/* Print text in statusbar. */
void UIStatus (const char * text, int delay, int warning, int ypos) {
	int i;
	
	attron (WA_REVERSE);
	if (warning)
		attron (COLOR_PAIR(10));
	
	/* Clear statusbar. */
	for (i = 0; i < COLS; i++) {
		mvaddch (ypos, i, ' ');
	}
	
	mvaddnstr (ypos, 1, text, COLS-2);
	
	/* attroff is called here. If the calling function had it switched on,
	   switch it on again there! */
	if (warning)
		attroff (COLOR_PAIR(10));
	attroff (WA_REVERSE);
		
	refresh();
	
	if (delay)
		sleep (delay);
}

/* Draw a box with WA_REVERSE at coordinates x1y1/x2y2 */
void UISupportDrawBox (int x1, int y1, int x2, int y2) {
	int i, j;
	
	attron (WA_REVERSE);
	for (i = y1; i <= y2; i++) {
		for (j = x1; j <= x2; j++) {
				mvaddch (i, j, ' ');
		}	
	}
	attroff (WA_REVERSE);
}

/* Draw main program header. */
void UISupportDrawHeader (const char * headerstring) {
	UISupportDrawBox(0, 0, COLS, 0);
	attron (WA_REVERSE);

	mvprintw (0, 1, "* Snowlog %s", VERSION);
	if (headerstring != NULL) {
		if (strlen(headerstring) > COLS-18)
			mvaddnstr (0, 19, headerstring, COLS-20);
		else
			mvaddstr (0, COLS-strlen(headerstring)-1, headerstring);
	}
	attroff (WA_REVERSE);
}

void SnowClearScreen (int * forceredraw) {
	if (*forceredraw) {
		clear();
		*forceredraw = 0;
	} else {
		move(0,0);
		clrtobot();
	}
}

void SnowClearLine (int y) {
	move (y, 0);
	clrtoeol();
}

void SnowStartColors (void) {
	start_color();

	/* The following call will automagically implement -1 as the terminal's
	default color for fg/bg in init_pair. */
	use_default_colors();

	/* Internally used color pairs.
	Use with WA_BOLD to get bright color (orange->yellow, gray->white, etc). */
	init_pair (SNOW_COLOR_RED, 1, -1);		/* red */
	init_pair (SNOW_COLOR_GREEN, 2, -1);	/* green */
	init_pair (SNOW_COLOR_ORANGE, 3, -1);	/* orange */
	init_pair (SNOW_COLOR_BLUE, 4, -1);		/* blue */
	init_pair (SNOW_COLOR_MAGENTA, 5, -1);	/* magenta */
	init_pair (SNOW_COLOR_CYAN, 6, -1);		/* cyan */
	init_pair (SNOW_COLOR_GREY, 7, -1);		/* gray */

	/* Default terminal color color pair */
	init_pair (63, -1, -1);
}

/* Take a URL and execute in default browser.
   Apply all security restrictions before running system()! */
void UISupportURLJump (const char * url) {
	int len;
	char *tmp;
	char *escapetext = NULL;
	char *syscall = NULL;
	
	/* Should not happen. Nope, really should not happen. */
	if ((url == NULL) || (strcmp(url, "-") == 0))
		return;
	
	/* Complain loudly if browser string contains a single quote. */
	if (strstr(browser, "'") != NULL)
		UIStatus ("Unsafe browser string (contains quotes)! See snownews.kcore.de/faq#toc2.4", 5, 1, LINES-1);
	
	/* Discard url if it contains a single quote! */
	if ((strstr(url, "'")) == NULL) {
		len = strlen(url) + 3;
		escapetext = malloc (len);
		snprintf (escapetext, len, "\'%s\'", url);
		len = strlen(browser) + len + 1;
		syscall = malloc (len);
		snprintf (syscall, len, browser, escapetext);
		free (escapetext);
	} else {
		UIStatus ("Invalid URL passed. Possible shell exploit attempted!", 3, 1, LINES-1);
		free (syscall);
		return;
	}
	
	len = strlen(syscall) + strlen("Executing %s") + 1;
	tmp = malloc (len);
	snprintf (tmp, len, "Executing %s", syscall);
	UIStatus (tmp, 0, 0, LINES-1);
	free (tmp);
	
	len = len + 13;
	tmp = malloc (len);
	snprintf (tmp, len, "%s 2>/dev/null", syscall);

	/* Switch on the cursor. */
	curs_set(1);

	system(tmp);

	InitCurses();

	/* Hide cursor again. */
	curs_set(0);

	free (tmp);
	free (syscall);
}

void Help (void) {
	int uiinput;
	int centerx, centery;
	int offset = 22;
	int offsetstr = 0;
	
	centerx = COLS / 2;
	centery = LINES / 2;
	
	UISupportDrawBox ((COLS/2)-25, (LINES/2)-6, (COLS/2)+26, (LINES/2)+6);
	
	attron(WA_REVERSE);
	mvaddstr (centery-5, COLS/2-strlen("Keyboard commands")/2, "Keyboard commands");
	mvaddstr (centery-3, centerx-offset, "tab:");
	mvaddstr (centery-2, centerx-offset, "n, p, up, down:");
	mvaddstr (centery-1, centerx-offset, "N, P, <-, ->:");
	mvaddstr (centery, centerx-offset, "space, b, pdown, pup:");
	mvaddstr (centery+1, centerx-offset, "<, >, home, end:");
	mvaddstr (centery+2, centerx-offset, "o:");
	mvaddstr (centery+3, centerx-offset, "O:");
	mvaddstr (centery+4, centerx-offset, "f:");
	mvaddstr (centery+5, centerx-offset, "s:");
/*	mvaddstr (centery+6, centerx-offset, "I:");*/
	
	mvaddstr (centery-3, centerx-offsetstr, "Search");
	mvaddstr (centery-2, centerx-offsetstr, "Next, previous item");
	mvaddstr (centery-1, centerx-offsetstr, "Next, previous host");
	mvaddstr (centery, centerx-offsetstr, "Next, previous page");
	mvaddstr (centery+1, centerx-offsetstr, "First, last item");
	mvaddstr (centery+2, centerx-offsetstr, "Open referer in browser");
	mvaddstr (centery+3, centerx-offsetstr, "Open current request");
	mvaddstr (centery+4, centerx-offsetstr, "Filters");
	mvaddstr (centery+5, centerx-offsetstr, "Request is Spam");
/*	mvaddstr (centery+6, centerx-offsetstr, "Show Spam");*/
	attroff(WA_REVERSE);

	UIStatus ("Press the any(tm) key to exit help screen.", 0, 0, LINES-1);
	uiinput = getch();
	
	/* Return input back into input queue so it gets automatically
	   executed. */
	if ((uiinput != '\n') && (uiinput != 'h') && (uiinput != 'q'))
		ungetch(uiinput);
}

/*
void DisplayHelpBox (char ** items) {

}
*/

static inline unsigned snow_stringhash__hashvalue(char const * const string)
{
    unsigned len = strlen(string);
    unsigned i, h = 0;
    for (i=0; i<len; i++) {
        h = 31*h + string[i];
    }
    return h;
} 

struct snow_stringhash *snow_stringhash_init(unsigned const table_size)
{
    struct snow_stringhash *newhash = malloc(sizeof(struct snow_stringhash));
    newhash->table_size = table_size;
    newhash->table = malloc(sizeof(struct snow_stringhash_item *)*table_size);
    memset(newhash->table, 0, sizeof(struct snow_stringhash_item *)*table_size);
    return newhash;
}

void snow_stringhash_free(struct snow_stringhash * const hash)
{
    struct snow_stringhash_item **table_ptr = hash->table;
    struct snow_stringhash_item *cur_item, *next_item;
    unsigned i;

    /* Iterate through the whole table and free all stored items */
    for (i = 0, table_ptr = hash->table;
         i < hash->table_size; 
         i++, table_ptr++) 
    {
        for (cur_item = *table_ptr; cur_item != NULL; cur_item = next_item) {
            next_item = cur_item->next;
            free(cur_item->string);
            free(cur_item);
        }
    }
    
    free(hash->table);
    free(hash);
}

char const * snow_stringhash_unique(struct snow_stringhash * const hash,
                                    char const * const string)
{
    struct snow_stringhash_item **table_ptr;
    struct snow_stringhash_item *cur_item, *new_item;
    
    table_ptr = hash->table + (snow_stringhash__hashvalue(string) % hash->table_size);
    
    // Check if the string is already in the hash
    for (cur_item=*table_ptr; cur_item != NULL; cur_item = cur_item->next) {
        if (strcmp(cur_item->string, string) == 0)
            return cur_item->string;
    }
    
    // String is not in the hash, add it
    new_item = malloc(sizeof(struct snow_stringhash_item));
    new_item->next = *table_ptr;
    new_item->string = strdup(string);
    *table_ptr = new_item;
    
    return new_item->string;
}

void snow_stringhash_dump(struct snow_stringhash * const hash)
{
    struct snow_stringhash_item **table_ptr = hash->table;
    struct snow_stringhash_item *cur_item;
    unsigned *bucket_sizes;
    unsigned cur_bucket;
    unsigned item_count = 0;
    
    bucket_sizes = malloc(sizeof(unsigned) * hash->table_size);
    memset(bucket_sizes, 0, sizeof(unsigned) * hash->table_size);
    
    /* Iterate through the whole table and collect the bucket sizes */
    for (cur_bucket = 0, table_ptr = hash->table;
         cur_bucket < hash->table_size; 
         cur_bucket++, table_ptr++) 
    {
        for (cur_item = *table_ptr; cur_item != NULL; cur_item = cur_item->next) {
            bucket_sizes[cur_bucket]++;
            item_count++;
        }
        fprintf(stderr, "Size of bucket %4d = %d: ", cur_bucket, bucket_sizes[cur_bucket]);
        for (cur_item = *table_ptr; cur_item != NULL; cur_item = cur_item->next) {
        	fprintf(stderr, "%s, ", cur_item->string);
        }
        fprintf(stderr, "\n");
    }
    
    free(bucket_sizes);
}
